/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is Yahoo! Mail Notifier code.
 *
 * The Initial Developer of the Original Code is
 * Michael Kraft.
 * Portions created by the Initial Developer are Copyright (C) 2005
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either of the GNU General Public License Version 2 or later (the "GPL"),
 * or the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */
 
var Cc = Components.classes;
var Ci = Components.interfaces;
var Cu = Components.utils;

Cu.import("resource://gre/modules/XPCOMUtils.jsm");

var gYm_notifier = {

	// default constants
	defaultloginURL		: "https://login.yahoo.com",
//	btloginURL 			: "https://edit.europe.yahoo.com",

	loginPATH			: "/config/login",
	verifyPATH 			: "/config/login_verify2",
	logoutPATH 			: "/config/login?logout=1",

	// This URL returns new mail count even if password expires (as long as user is logged in) 
//	defaultcheckURL		: "http://api2.my.yahoo.com/2.0/content/widgetcount",  // No longer works
//	defaultcheckURL2 	: "http://us.yahoo.com",	// doesn't work
//	altcheckURL			: "https://us.data.toolbar.yahoo.com/slv/v4/not",  // only returns at most 9 new unread
	defaultcheckURL		: "http://www.yahoo.com/jsal?_action=show&_subAction=get_vitals&channels=%5B%22%2Fuser%2Fymail%22%5D&_container=0&_id=p_30345316&_mode=json",  // Works, but not much known about it
	
	logFileName			: "Yahoo_Mail_Notifier_log.txt",
	
	// The first url below will reset the new mail count when it gets the folder unread count, the second one won't
//	mailFolderUnreadURL	: "http://us.mg1.mail.yahoo.com/ws/mail/v1/formrpc?m=ListFolders&appid=YahooMailRC&resetMessengerUnseen=true",
	mailFolderURL		: "http://us.mg1.mail.yahoo.com/ws/mail/v1/formrpc?m=ListFolders&appid=YahooMailRC",
 
	defaultmailURL 		: "http://mail.yahoo.com",
	defaultcookieDomain : "yahoo.com",
	
	// current password for default user
	currentDefaultPassword : null,
	
	// Is this an older browser based off of buggy FF 1.0.7 code?
	oldBrowser : true, 			// default to true
	
	loginFailCount : 0,
	checkFailCount : 0,
	newMailCount : 0,
	allowCheck : false,			// windows default to not allowed to check

	mouseEvtTime : 0,			// save time of click event.
	mouseId : null,				// handle of click setTimeOut
	timer : null,				// NsiTimer object
	sound : null,				// NsiSound object
	sendRetryId : null,			// handle http retry setTimeout for login
	sendRetry2Id : null, 		// handle http retry setTimeout for checkmail
	sendRetry3Id : null, 		// handle http retry setTimeout for getMailFolderCount

	folderFailCount : 0, 		// Fail count when checking folders (try 3 times)

	xmlHttpReq : null, 			// XMLHttpRequest object
	logFile : null,				// Log File
	
	Mozilla_1_9 : false,		// True if this browser was built off the Mozilla 1.9 branch
	Mozilla_1_9_2_plus : false,	// True if this browser was off the Mozilla 1.9 branch or higher
	
	cookieValue: [],   // Since cookie isn't returned in private browsing need to store it temporarily
	
	//
	// Initialize stuff when window is opened
	//
	init : function() {
		// these can change based of user's email
		this.loginURL 		= this.defaultloginURL;
		this.checkURL 		= this.defaultcheckURL;
		this.mailURL		= this.defaultmailURL;
		this.cookieDomain 	= this.defaultcookieDomain;
		
		 // EOL for log file (based on O/S)
		this.mEOL = /win|os[\/_]?2/i.test(navigator.platform)?"\r\n":/mac/i.test(navigator.platform)?"\r":"\n";	
		
		// Delete log file if not logging or too large
		this.deleteLogFile();
	},

	//
 	// do stuff when new browser window is loaded
 	// use a settimeout to allow window to open if masterpassword is set
 	//
	initialize : function() {
		if (document.getElementById("ym-notifier-statusbar-item")) { 
			window.addEventListener('unload', gYm_notifier.uninitialize, false);
			window.setTimeout(gYm_notifier.initialize2, 5);
		}
		window.removeEventListener('load', gYm_notifier.initialize, false);
	},

	initialize2 : function() {
		var showStatusbar = (ym_prefs.getPref(ym_prefs.PREF_STATUSBAR) == ym_prefs.CONST_ALWAYS);
		var myStatusbar = document.getElementById("ym-notifier-statusbar-item");
		if (myStatusbar) myStatusbar.setAttribute("collapsed", !showStatusbar);

		// Firefox 1.0.7 based browsers suffer from bug 
		// https://bugzilla.mozilla.org/show_bug.cgi?id=174320
		// so we need to know if this is an older browser or not.  linkedBrowser
		// only exists in browsers based off the FF 1.5 and SeaMonkey code.
		try {
			if (document.getElementById("content").mCurrentTab.linkedBrowser) gYm_notifier.oldBrowser = false;
		} catch (ex) {}
		gYm_notifier.ym_log("Old Browser = " + gYm_notifier.oldBrowser);
		
		try {
			// Add observer for when logging in and out and for changing new mail indicator
			var observerService = Cc["@mozilla.org/observer-service;1"].getService(Ci.nsIObserverService);
			observerService.addObserver(gYm_notifier,"ym-gui-change",false); 	
			ym_prefs.addObserver(ym_prefs.PREF_STATUSBAR, gYm_notifier);

			// Is this the first browser window?
			var otherChecking = false;
			var wm = Cc['@mozilla.org/appshell/window-mediator;1'].getService(Ci.nsIWindowMediator);
			var browsers=wm.getEnumerator('navigator:browser');
			while (browsers.hasMoreElements()) {
				var win = browsers.getNext(); 
				if (win.gYm_notifier && win.gYm_notifier.allowCheck) {
					otherChecking = true;
					break;
				}
			}

			// This is the first window
			if (!otherChecking) {
				gYm_notifier.ym_log("New window allowed to check");
				
				// allow this window to check
				gYm_notifier.allowCheck = true;
				
				// listen for interval changes and start checking for email if set to do so
				ym_prefs.addObserver(ym_prefs.PREF_INTERVAL, gYm_notifier);
				
				// Listen for cookie changes (for when logged in/out)
				observerService.addObserver(gYm_notifier, "cookie-changed", false);
				observerService.addObserver(gYm_notifier, "private-cookie-changed", false);
				
				if (ym_prefs.getPref(ym_prefs.PREF_AUTO_CHECK)) {
					var loginState = gYm_notifier.loginState();
					gYm_notifier.ym_log("loginState : " + loginState);
					if (loginState < 4) {
						gYm_notifier.doLogin();
					}
					else {
						gYm_notifier.startChecking();
					}
				}
			}
			else {
				gYm_notifier.ym_log("New window not allowed to check");

				// set icons, menus and tooltips to the same as other windows
				var masterStatusbar = win.document.getElementById("ym-notifier-statusbar-item");

				// get new mail count from tooltip
				if (masterStatusbar) {
					var messageCount = masterStatusbar.getAttribute("label");
					var lastCheckTime = masterStatusbar.getAttribute("lastCheckTime"); 
					if (masterStatusbar.getAttribute("error") == "true") gYm_notifier.updateGUI("check-error");
					else if (masterStatusbar.getAttribute("logged-in") == "true") {
						if (messageCount) gYm_notifier.updateGUI(messageCount + "  " + lastCheckTime);
						else gYm_notifier.updateGUI("0  " + lastCheckTime);
					}
				}
			}
			
			// Check to see if might need to work around 3rd party cookies (bug 437174) - only in Firefox 3.0
			var appInfo = Cc["@mozilla.org/xre/app-info;1"].getService(Ci.nsIXULAppInfo);
			var vc = Cc["@mozilla.org/xpcom/version-comparator;1"].getService(Ci.nsIVersionComparator);
			gYm_notifier.Mozilla_1_9 = ((vc.compare(appInfo.platformVersion, "1.9") >= 0) && (vc.compare(appInfo.platformVersion, "1.9.1") < 0));
			gYm_notifier.Mozilla_1_9_2_plus = (vc.compare(appInfo.platformVersion, "1.9.2") >= 0);
		}
		catch (ex) {
			dump(ex);
			gYm_notifier.ym_logError(ex, 1);
		}
	},
	 
	// 
	// do stuff when browser window closes
	//
	uninitialize : function() {
		try {
			gYm_notifier.ym_log("Uninitializing");
			// Remove observers
			var observerService = Cc["@mozilla.org/observer-service;1"].getService(Ci.nsIObserverService);
			observerService.removeObserver(gYm_notifier,"ym-gui-change");	 
			ym_prefs.removeObserver(ym_prefs.PREF_STATUSBAR, gYm_notifier);
			ym_prefs.removeObserver(ym_prefs.PREF_INTERVAL, gYm_notifier);
			
			// Listen for cookie changes (for when logged in/out)
			observerService.removeObserver(gYm_notifier, "cookie-changed");
			observerService.removeObserver(gYm_notifier, "private-cookie-changed");
			

			window.removeEventListener('unload', gYm_notifier.uninitialize, false);
			
			var startNewTimer = false;

			// If this window has a timer, stop it (should only happen for polling window)
			if (gYm_notifier.timer) {
				gYm_notifier.stopChecking();
				startNewTimer = true;
			}
		
			// Is this window polling, if so get another window to do it
			// A component would be better, do that later
			if (gYm_notifier.allowCheck) {
				// Make sure request object is gone
				delete(gYm_notifier.xmlHttpReq);
				gYm_notifier.xmlHttpReq = null;
		
				// Find another window to take over polling job
				var newYmNotifier = null;
				var wm = Cc['@mozilla.org/appshell/window-mediator;1'].getService(Ci.nsIWindowMediator);
				var browsers=wm.getEnumerator('navigator:browser');
				if(browsers.hasMoreElements()) { 
					var win = browsers.getNext();
					if (window == win) win = browsers.getNext();
					newYmNotifier = win.gYm_notifier;
					ym_prefs.addObserver(ym_prefs.PREF_INTERVAL, newYmNotifier);
			
					// Listen for cookie changes (for when logged in/out)
					observerService.addObserver(newYmNotifier, "cookie-changed", false);
					observerService.addObserver(newYmNotifier, "private-cookie-changed", false);
			
					// Allow new window to check
					newYmNotifier.allowCheck = true;
			
					// Copy current default password to new window
					newYmNotifier.currentDefaultPassword = gYm_notifier.currentDefaultPassword;
				
					// Copy current mail count to new window
					newYmNotifier.newMailCount = gYm_notifier.newMailCount;
					
					gYm_notifier.ym_log("Closed window gave checking permission to another window.");
				}
				// last window closing, check to see if user does not want to remember password and if so logout
				// to remove yahoo cookie
				else {
					// TBD MULTI-USER SUPPORT
					var username = ym_prefs.getPref(ym_prefs.PREF_DEFAULT_USER);
					if (username) {
						ym_prefs.loadUser(username, false);
						if (ym_prefs.user.userName && !ym_prefs.user.remember_pw) gYm_notifier.logout(username);
					}
				}
			}
			// Tell new window to start checking for mail but to not actually check at this moment. The reason 
			// for this is that if checkNow is called from this window,  the XMLHttpRequest component 
			// will generate an error.	I do not know why this happens, but it does so just don't call it.
			// This may cause a slight delay in checking for mail.	Once the component is implemented this 
			// won't be a problem.
			if (newYmNotifier && startNewTimer) newYmNotifier.startChecking(true);
		}
		catch (ex) {
			gYm_notifier.ym_logError(ex, 1);
		}
		finally {
			gYm_notifier = null;
		}
	},
	 
	//
	// start checking for new mail
	//
	startChecking : function(dontCheckNow) {
		this.ym_log("begin startChecking");
	
		if (!this.allowCheck) {
			this.ym_log("Window not allowed to check");
			return;
		}

		if (this.timer) {
			this.timer.cancel();
			this.ym_log("restarting timer");
		}
		// this needs to be done even if the timer is cancel because of bug 325418 otherwise time will be off
		// if user changes interval
		// https://bugzilla.mozilla.org/show_bug.cgi?id=325418
		this.timer = Cc["@mozilla.org/timer;1"].createInstance(Ci.nsITimer);

		this.timer.init(gYm_notifier, ym_prefs.getPref(ym_prefs.PREF_INTERVAL) * 60000, Ci.nsITimer.TYPE_REPEATING_PRECISE);
 
		// By default, check now otherwise it will start checking after the timer expires
		if (!dontCheckNow) {
	
			// Firefox 1.0.7 based browsers suffer from https://bugzilla.mozilla.org/show_bug.cgi?id=174320
			// Work around is to use settimeout
			if (this.oldBrowser) window.setTimeout(gYm_notifier.checkMail, 0)
			else gYm_notifier.checkMail();
		}
 
		this.ym_log("leave startChecking");
	},

	//
	// stop checking for new mail
	//
	stopChecking : function() {
		this.ym_log("stopChecking");
		if (this.timer) {
			this.ym_log("timer canceled");
			this.timer.cancel();
			this.timer = null;
		}
		if (this.xmlHttpReq) this.xmlHttpReq.abort();
	},

	//
	// log out of Yahoo! by deleting cookie
	//
	logout : function(username, usernameLoggingIn) {
		try {
			// if no username specified, log out default user  // TBD MULTI-USER SUPPORT
			if (!username) {
				username = ym_prefs.getPref(ym_prefs.PREF_DEFAULT_USER);
			}
			this.ym_log("logging out " + username);
		
			// make sure if logging in that that user's cookie domain is cleared
			var otherCookieDomain = null;
			if (usernameLoggingIn) {
				this.findYahooURL(usernameLoggingIn);
				otherCookieDomain = this.cookieDomain;
			}
				
			// set correct URLs
			this.findYahooURL(username);
		
			// remove stored cookie
			this.cookieValue = [];
		
			// logout by deleting login cookie
			var cookMan = Cc["@mozilla.org/cookieService;1"].getService(Ci.nsICookieManager);
			
			if (!otherCookieDomain || (otherCookieDomain == this.cookieDomain)) {
				cookMan.remove("." + this.cookieDomain, "Y", "/", false);
			
				// clear stored cookie
				ym_prefs.saveUserCookie("", username);
				
				// if default user forget default password and actually log out	 // TBD MULTI-USER SUPPORT
				if (username == ym_prefs.getPref(ym_prefs.PREF_DEFAULT_USER)) {
					this.currentDefaultPassword = null;

					// notify all windows of logout
					var observerService = Cc["@mozilla.org/observer-service;1"].getService(Ci.nsIObserverService);
					observerService.notifyObservers(null,"ym-gui-change","logout " + username);
				}
			}
			else if (otherCookieDomain) cookMan.remove("." + otherCookieDomain, "Y", "/", false);
		}
		catch (ex) {
			this.ym_logError(ex, 1);
		}
	},

	//
	// Manual check for email or login if not already logged in
	//
	manualCheckMail : function () {
		var loginState = gYm_notifier.loginState();
		this.ym_log("manualCheckMail\nloginState : " + loginState);
		if (loginState < 4) {
			gYm_notifier.doLogin()
		}
		else {
			gYm_notifier.checkMail();
		}
	},

	//
	// Find correct URL to use if user specifies email address
	// Currently this is needed only for BT Yahoo and Yahoo Japan as 
	// they are the only two which won't work.	Ignore the email for others
	// since it will break.  Yahoo will allow the user to login with their
	// email address so that can be used as the login name.
	///
 
	findYahooURL : function(username) {
	
		// if there is not a defined username use the default one  // TBD MULTI-USER SUPPORT
		if (!username) username = ym_prefs.getPref(ym_prefs.PREF_DEFAULT_USER);

		// set default URLS
		this.cookieDomain = this.defaultcookieDomain;
		this.mailURL      = this.defaultmailURL;
		this.loginURL     = this.defaultloginURL;
		this.checkURL     = this.defaultcheckURL;

		// find @ and ., if this is found its an email address
		var atSymbol = username.indexOf("@");
		var lastdot = username.lastIndexOf(".");

		// if username is actually an email address, strip off the username and find the login server
		if ((atSymbol != -1) && (lastdot != -1)) {

			// Yahoo! Japan uses different URLS
			if (username.indexOf("@yahoo.co.jp") != -1) { 		
				this.cookieDomain = "yahoo.co.jp";
				this.loginURL 		= "https://login.yahoo.co.jp";
				this.checkURL 		= "http://www.yahoo.co.jp";
				this.mailURL		= "http://mail.yahoo.co.jp";
			}
		}
	
		//this.ym_log("loginURL = " + this.loginURL + ", checkURL = " + this.checkURL + ", mailURL = " + 
		//						this.mailURL + ", cookieDomain = " + this.cookieDomain);
	},

	// 
	// Login (if necessary) and begin polling
	//
	doLogin : function(aForce) {
		this.ym_log("doLogin");
	
		// A better way would be to observe changes to the Yahoo cookies and automatically clear
		// the stored cookie if user logs out at web site, but this works for now.	A minor problem
		// is that Yahoo will keep asking to verify the password even though we are logged in.	
		// Observing when the new cookie is stored in the browser and storing it into the user 
		// would lessen this problem.  Really we should "verify" the login at least once every
		// 24 hours.
		// The current work around is to "forget" the login cookie if over 12 hours has passed
		// since the user last actually logged (didn't use cookie)

		// TBD MULTI-USER SUPPORT
		var username = ym_prefs.getPref(ym_prefs.PREF_DEFAULT_USER);
		if (username) {
			ym_prefs.loadUser(username);
			if (ym_prefs.user.userName) {
				var password = ym_prefs.user.password; 
				var thecookie = ym_prefs.user.cookie;
				var useCookie = ym_prefs.user.useCookie;

				var loginState = this.loginState(username);
				// If want to force a login via Yahoo
				if (aForce) useCookie = false;
			
				// set correct URLs
				this.findYahooURL(username);

				// only log in using cookies, if not logged in as someone else 
				if (useCookie && thecookie && (loginState != 1) &&
				    this.getCookie("." + this.cookieDomain, 'T')) {
					this.loginWithCookie(username, thecookie);
					return;
				}
				else if (password) {
					this.login(username, password);
					return;
				}
				// temporarily stored password for default user
				else if (this.currentDefaultPassword) {
					this.login(username, this.currentDefaultPassword);
					return;
				}
			}
		}
		// either no user or no password so open login window
		this.openLoginWindow();
	},

	//
	// log in to Yahoo! using stored cookie info
	//
	loginWithCookie : function(username, cookieValue) {

		this.ym_log("Logging in with cookie " + cookieValue);

		var ioService = Cc['@mozilla.org/network/io-service;1'].getService(Ci.nsIIOService);
		var cookieService = Cc["@mozilla.org/cookieService;1"].getService(Ci.nsICookieService);
		var observerService = Cc["@mozilla.org/observer-service;1"].getService(Ci.nsIObserverService);
	
		// set correct URLs
		this.findYahooURL(username);
		
		var uri = ioService.newURI("http://" + this.cookieDomain, null, null);
		var cookieString = "Y=" + cookieValue + ";domain=." + this.cookieDomain + 
							 ";path=/;expires=Thursday, April 15, 2010 3:57:58 PM;"
		cookieService.setCookieString(uri, null, cookieString, null);

		observerService.notifyObservers(null,"ym-gui-change","login " + username);
		gYm_notifier.loginFail = false;
	},

	// 
	// log in to Yahoo! if necessary
	//
	login : function(username, pword, persist, challenge, challengeTry) {
		this.ym_log("login start");
		
		// set correct URLs
		this.findYahooURL(username);
	
		var theUserName = username;
		var loginState = this.loginState(username);
		var theURL = this.loginURL + this.loginPATH;
		var persistent = "&.persistent=Y";
		var challengeParam = "";
		var extraParams = "";
		var verify = false;
	
		if (persist==false || loginState == 2 || loginState == 4) persistent = "";
		if (challenge && challenge != "undefined") challengeParam = "&.challenge=" + challenge;
	
		// find @ and ., if this is found its an email address
		var atSymbol = username.indexOf("@");
		var lastdot = username.lastIndexOf(".");
		
		// if username is actually an email address, check for partners
		if ((atSymbol != -1) && (lastdot != -1)) {
	
			// Add params for loging in with BT Internet
			if ((username.indexOf("@btinternet.com") != -1) ||
			    (username.indexOf("@talk21.com") != -1) ||
			    (username.indexOf("@btopenworld.com") != -1))
				extraParams = "&.intl=uk&.partner=bt-1&.src=bt-1";
	
			// Add params for loging in with SBCGlobal/AT&T
			else if ((username.indexOf("@sbcglobal.net") != -1) ||
				     (username.indexOf("@att.net") != -1))
				extraParams = "&.intl=us&.partner=sbc&.src=sbc";
	
			// Add params for loging in with Verizon
			else if (username.indexOf("@verizon.net") != -1)
				extraParams = "&.intl=us&.partner=vz-acs&.src=vz-acs";
	
			// Yahoo! Japan won't work if username is email address
			else if (username.indexOf("@yahoo.co.jp") != -1) 
				theUserName = username.substring(0,atSymbol);
		}
	
		this.ym_log("Logging in as " + theUserName + " with persist = " + persist + 
					" and challenge = " + challenge + ". Loginstate = " + loginState);
	
		if (loginState == 1) {
			this.ym_log("Logging out first because logged in as wrong user.");
			this.logout(null,username);
		}
		else if (loginState > 1) {
			verify = true;
			theURL = this.loginURL + this.verifyPATH;
			this.ym_log("User was already logged in, verifying login.");
		}
		this.ym_log("Login URL: " + theURL);

		try {
			// initialize XMLHttpRequest Object to allow 3rd party cookies
			if (!this.xmlHttpReq) {
				this.ym_log("xmlHttpReq not created yet, so creating it.");
				this.xmlHttpReq = new XMLHttpRequest();
			}
			this.xmlHttpReq.open("POST", theURL, true);
			this.xmlHttpReq.channel.loadFlags |= Ci.nsIRequest.LOAD_BYPASS_CACHE;
			this.workAroundBug437174(this.xmlHttpReq);
			this.privateBrowsingCheck(this.xmlHttpReq);
			this.xmlHttpReq.setRequestHeader('Content-Type', 'application/x-www-form-urlencoded');
			this.xmlHttpReq.setRequestHeader('Cache-Control','no-cache, must-revalidate, proxy-revalidate');
			this.xmlHttpReq.setRequestHeader('Pragma','no-cache, must-revalidate, proxy-revalidate');
			this.xmlHttpReq.onerror = function() {
				gYm_notifier.ym_log("login Onerror fired with status of " + gYm_notifier.xmlHttpReq.status +
				                    " and readyState of " + gYm_notifier.xmlHttpReq.readyState);
				gYm_notifier.xmlHttpReq.abort();
				gYm_notifier.xmlHttpReq = null;  // needed or send won't do anything
				gYm_notifier.loginFailCount = gYm_notifier.loginFailCount + 1;
				if (gYm_notifier.loginFailCount > 2) {
					gYm_notifier.loginFailCount = 0;
					// if login was trigger automatically (with no user action) it must have been while 
					// checking so change the icon and put up error tooltip, otherwise put up alert window
					if (gYm_notifier.timer) {
						var observerService = Cc["@mozilla.org/observer-service;1"].getService(Ci.nsIObserverService);
						observerService.notifyObservers(null,"ym-gui-change","check-error " + username);
					}
					else alert('"' + username + '" - ' + gYm_notifier.getString("loginfail")); 
				}
				else {
					gYm_notifier.login(username, pword, persist);
				}
				this.postData = null;
			}
			this.xmlHttpReq.onload = function() {
				if (gYm_notifier.xmlHttpReq.readyState==4) {
					if (gYm_notifier.xmlHttpReq.status == 200) {
						gYm_notifier.loginFailCount = 0;
						var observerService = Cc["@mozilla.org/observer-service;1"].getService(Ci.nsIObserverService);
						// A successful login should contain one of the two lines below (2nd is for Yahoo Japan): 
						// window.location.replace("http://my.yahoo.com");
						// window.location.replace("http://www.yahoo.co.jp");
						// Sometimes Yahoo redirects the user to a question page and the login is placed on 
						// hold so we should detect that and open a page containing the redirected page.
						if (ym_prefs.getPref(ym_prefs.PREF_LOG_ENABLED)) {
						 	gYm_notifier.ym_write_log((new Date).toGMTString() + ':\nLogin Result: ' + gYm_notifier.xmlHttpReq.responseText + '\n');
						}
						//PLACEHOLDER
						if (gYm_notifier.xmlHttpReq.responseText.indexOf('window.location.replace') != -1) {
							if (gYm_notifier.loginState(username) > 0) {
								// notify all windows of login
								observerService.notifyObservers(null,"ym-gui-change","login " + username);
								gYm_notifier.loginFail = false;
								// save password if this was default user  // TBD MULTI-USER SUPPORT
								if (username == ym_prefs.getPref(ym_prefs.PREF_DEFAULT_USER)) {
									gYm_notifier.currentDefaultPassword = pword;
								}
							}
						}
						// Check to see if need to verify login
						else if (!verify && gYm_notifier.xmlHttpReq.responseText.indexOf("login_verify2") != -1) {
							gYm_notifier.ym_log("login: Password expired, need to verify login.");
							// clear stored cookie
							ym_prefs.saveUserCookie("", username);
							// Firefox 1.0.x requires the following line or it won't work for some reason
							if (gYm_notifier.oldBrowser) gYm_notifier.xmlHttpReq = null;
							gYm_notifier.login(username, pword, persist);
						}
						// Bad password
						else if (gYm_notifier.xmlHttpReq.responseText.indexOf('name=".tries"') != -1) {
							gYm_notifier.ym_log("Login didn't work.");
							// BT Internet needs a challenge parameter so this is the work around.
							// Sometimes it doesn't work so we try twice.
							if (!challengeParam || challengeParam=="undefined" || challengeTry=="first") {
								var checkText = '".challenge" value="';
								var tryText = "first";
								if (challengeTry=="first") tryText = "second";
								var sChallenge = gYm_notifier.xmlHttpReq.responseText.indexOf(checkText);
								if (sChallenge != -1) {
									var eChallenge = gYm_notifier.xmlHttpReq.responseText.indexOf
												 	('"', sChallenge + checkText.length);
									if (eChallenge != -1) {
										var challengeText = gYm_notifier.xmlHttpReq.responseText.substring
														 	(sChallenge + checkText.length, eChallenge);
										gYm_notifier.ym_log("login failed, trying " + tryText + " challenge = " + challengeText);
										// Firefox 1.0.x requires the following line or it won't work for some reason
										if (gYm_notifier.oldBrowser) gYm_notifier.xmlHttpReq = null;
										gYm_notifier.login(username, pword, persist, challengeText, tryText);
									}  
								}
							}
							// Yahoo wants user to fill in CAPTCHA so display it
							else if (gYm_notifier.xmlHttpReq.responseText.indexOf('id="secword"') != -1) {
								gYm_notifier.ym_log("Yahoo returned a CAPTCHA.");
								gYm_notifier.openResponseTab();
							}
							// Else give up and display error
							else {
								gYm_notifier.ym_log("Login failed too many times, giving up.");
								alert ('"' + username + '" - ' + gYm_notifier.getString("baduser"));
								gYm_notifier.loginFail = true;
								observerService.notifyObservers(null,"ym-gui-change","logout " + username);
							}
						}
						// Yahoo logged in, but wants something so prompt user.
						else {
							// Pop up window with Yahoo's response since it wants something
							gYm_notifier.ym_log("Yahoo returned something unexpected.");
							gYm_notifier.openResponseTab();
							
							// notify all windows of login
							observerService.notifyObservers(null,"ym-gui-change","login " + username);
							gYm_notifier.loginFail = false;
							// save password if this was default user  // TBD MULTI-USER SUPPORT
							if (username == ym_prefs.getPref(ym_prefs.PREF_DEFAULT_USER)) {
								gYm_notifier.currentDefaultPassword = pword;
							}
						}
					}
					else {
						gYm_notifier.loginFailCount = gYm_notifier.loginFailCount + 1;
						if (gYm_notifier.loginFailCount > 2) {
							gYm_notifier.loginFailCount = 0;
							// if login was trigger automatically (with no user action) it must have been while 
							// checking so change the icon and put up error tooltip, otherwise put up alert window
							if (gYm_notifier.timer) {
								var observerService = Cc["@mozilla.org/observer-service;1"].getService(Ci.nsIObserverService);
								observerService.notifyObservers(null,"ym-gui-change","check-error " + username);
							}
							else alert('"' + username + '" - ' + gYm_notifier.getString("loginfail"));
							gYm_notifier.xmlHttpReq.abort();
						}
						else {
							gYm_notifier.login(username, pword, persist);
						}
					}
					this.postData = null;
				} 
			}
			// If xmlHttpReq is ready, send request
			if (this.xmlHttpReq.readyState == 1) {
				// if retrying clear pending retry
				if (this.sendRetryId) {
					clearTimeout( this.sendRetryId ); 	// Clear pending retry	
					this.sendRetryId = null;
				} 		
				
				// Send the request, even though done is set to this.defaultcheckURL it doesn't really
				// make a difference as long as it is a valid URL
				this.postData = 'login=' + encodeURIComponent(theUserName) + '&passwd=' + encodeURIComponent(pword) + 
				                persistent + challengeParam + extraParams;
				this.xmlHttpReq.send(this.postData);
				this.ym_log('?login=' + encodeURIComponent(theUserName) + '&passwd=XXXXXXXX' + 
				             persistent + challengeParam + extraParams);
			}
			// if it is not ready try again in 2 seconds
			else {
				this.ym_log("Login readyState = " + this.xmlHttpReq.readyState + ", retrying in 2 seconds.");
				this.xmlHttpReq.abort();
				this.sendRetryId = setTimeout(function() { gYm_notifier.login(username, pword, persist, challenge, challengeTry); }, 2000);
			}
		}
		catch (ex)
		{
			this.ym_logError(ex, 1);
		}
	},
	 
	//
	// This is used to display a tab with a response from Yahoo in the case where the login either fails because
	// Yahoo wants the user to enter a CAPTCHA or Yahoo logged in the user, but wants the user to do something else
	//
	openResponseTab: function() {
		this.ym_log("Opening Yahoo response in tab.");
//		var found = aText.match(/<body>((.|\n)*)<\/body>/mi); 
//		this.ym_log(found);
//		var aWin = window.open("", "Yahoo_Response_Window", "left=0, top=0, alwaysRaised, scrollbars, status, centerscreen");
//		aWin.resizeTo(window.screen.availWidth, window.screen.availHeight);
//		var newBody = aWin.document.createElement("body");
//		newBody.innerHTML = found[1];
//		aWin.document.body = newBody;

		var uri = this.xmlHttpReq.channel.originalURI.spec;
		this.privateBrowsingCheck(this.xmlHttpReq);
       
		var sis = Cc["@mozilla.org/io/string-input-stream;1"].createInstance(Ci.nsIStringInputStream);
		sis.data = this.postData;
       
		var postData = Cc["@mozilla.org/network/mime-input-stream;1"].createInstance(Ci.nsIMIMEInputStream);
		postData.addHeader("Content-Type", "application/x-www-form-urlencoded");
		postData.addContentLength = true;
		postData.setData(sis);
		
		var tabbrowser = top.document.getElementById("content");
		var tab = tabbrowser.addTab(uri, null, "", postData);
		tabbrowser.selectedTab = tab;
	},
	
	//
	// Grabs the list of folders and how much unread and total mail there is in the mailbox.	Not currently used.
	// If the this.mailFolderUnreadURL is used, it will also reset the new mail count when checking.
	// This currently isn't used at the moment.
	//
	getMailFolderCount : function(username, aURL) { 
		// Doesn't work for Yahoo Japan.
		if (username.indexOf("@yahoo.co.jp") != -1) return;
		var theURL = (aURL) ? aURL : gYm_notifier.mailFolderURL;
		gYm_notifier.ym_log('Getting mail folder info at ' + theURL);
		
		try {
			// Create a new XMLHttpRequest object so it doesn't interfere with checking for mail.
			var requester = new XMLHttpRequest();
			requester.open("GET", theURL, true);
			requester.channel.loadFlags |= Ci.nsIRequest.LOAD_BYPASS_CACHE;
			gYm_notifier.workAroundBug437174(requester);
			gYm_notifier.privateBrowsingCheck(requester);
			requester.setRequestHeader('Cache-Control','no-cache, must-revalidate, proxy-revalidate');
			requester.setRequestHeader('Pragma','no-cache, must-revalidate, proxy-revalidate');
			requester.onerror = function() {
				// An error occurred, notify user
				gYm_notifier.folderFailCount = gYm_notifier.folderFailCount + 1;
				gYm_notifier.ym_log("getMailFolderCount Onerror fired with status of " + requester.status +
				                    " and readyState of " + gYm_notifier.xmlHttpReq.readyState + ". Failed " + 
				                    gYm_notifier.folderFailCount + " times." +
									((gYm_notifier.folderFailCount>3)?"Giving Up":"Trying again in 2 seconds"));
				this.abort();
				if (gYm_notifier.folderFailCount > 3) {
					gYm_notifier.folderFailCount = 0;
				}
				else {
					gYm_notifier.sendRetry3Id = setTimeout(gYm_notifier.getMailFolderCount, 2000);
				}
			}
			requester.onload = function() {
				gYm_notifier.ym_log('getMailFolderCount request status (' + this.status + ')');
				if (this.readyState==4) {
					gYm_notifier.ym_log("ReponseText: " + this.responseText);
					gYm_notifier.folderFailCount = 0;
					if (this.status == 200) {
						gYm_notifier.pwExpired = false;
						var xmldoc = this.responseXML;
						var folders = xmldoc.getElementsByTagName('folder');
						var unread = 0;
						var total = 0;
						for (var i=0; i<folders.length; i++) {
							if ((folders.item(i).getAttribute("isSystem") == "false") || (folders.item(i).firstChild.getAttribute("name") == "Inbox")) {
								unread = unread + parseInt(folders.item(i).getAttribute("unread"));
								total = total + parseInt(folders.item(i).getAttribute("total"));
							}
						}
						gYm_notifier.ym_log("Folders: " + folders.length + ", Unread: " + unread + ", Total: " + total);
					}
					else if (this.status == 500) {
						var xmldoc = this.responseXML;
						var code = xmldoc.getElementsByTagName('code').item(0);
						if ((typeof(code) != "undefined") && code && code.firstChild) {
							// Password expired
							if (code.firstChild.data == "Client.ExpiredCredentials") {
								// Password has expired
								gYm_notifier.ym_log("getMailFolderCount: Password expired.");
								gYm_notifier.pwExpired = true;
								return;
							}
							// Need new session id
							else if (code.firstChild.data == "Client.ClientRedirect.SessionIdReissue") {
								var newURL = xmldoc.getElementsByTagName('url').item(0);
								if ((typeof(newURL) != "undefined") && newURL && newURL.firstChild) {
									gYm_notifier.ym_log("Redirection URL: " + newURL.firstChild.data);
									gYm_notifier.getMailFolderCount(username, newURL.firstChild.data);
								}
							}
						}
					}
					else {
						var xmldoc = this.responseXML;
						var reasonText = "none";
						if (xmldoc) {
						 	var message = xmldoc.getElementsByTagName('message').item(0);
						 	if ((typeof(message) != "undefined") && message && message.firstChild) reasonText = message.firstChild.data;
						}
						gYm_notifier.ym_log("Could not retrieve folder count. Reason: " + reasonText);
					}
				}
			}
			// If xmlHttpReq is ready, send request
			if (requester.readyState == 1) {
				// if retrying clear pending retry
				if (gYm_notifier.sendRetry3Id) {
					clearTimeout( gYm_notifier.sendRetry3Id );	 // Clear pending retry  
					gYm_notifier.sendRetry3Id = null;
				} 		
				requester.send(null);
			}
			// if it is not ready try again in 2 seconds
			else {
				gYm_notifier.ym_log("getMailFolderCount readyState = " + requester.readyState + ", retrying in 2 seconds.");
				requester.abort();
				gYm_notifier.sendRetry3Id = setTimeout(gYm_notifier.getMailFolderCount, 2000);
			}
		}
		catch (ex)
		{
			gYm_notifier.ym_logError(ex, 1);
		}
	},
 
	//
	// check for new mail
	// 	
	checkMail : function() {
	
		// Check to see if the same user is still logged in
		// TBD MULTI-USER SUPPORT
		var loginState = gYm_notifier.loginState();
		if (loginState <= 1) {
			gYm_notifier.ym_log('LoginState = ' + loginState + ', when checking for new mail, logging out');
		 	// logout to prevent problems
		 	gYm_notifier.logout();
			 
		 	// there should probably be some kind of alert here to tell
		 	// user that the extension logged back into default user.
		}
		
		// Check if its time to relogin to Yahoo before checking for mail
		// TBD MULTI-USER SUPPORT
		var username = null;
		ym_prefs.loadUser(null, false);
		if (ym_prefs.user.userName) {
		 	var useCookie = ym_prefs.user.useCookie;
		 	username = ym_prefs.user.userName;
		 	//gYm_notifier.getMailFolderCount(username);
		 	if (!useCookie && (username.indexOf("@yahoo.co.jp") != -1)) {
			 	gYm_notifier.ym_log('Time to relogin to Yahoo')
			 	gYm_notifier.doLogin();
			 	return;
		 	}
		}
	
		// set correct URLs
		gYm_notifier.findYahooURL(username);
			
		gYm_notifier.ym_log('Checking mail for ' + username + ' at ' + gYm_notifier.checkURL)

		// Initialize XMLHttpRequest object to allow cookies
		if (!gYm_notifier.xmlHttpReq) {
			gYm_notifier.ym_log("xmlHttpReq not created yet, so creating it.");
			gYm_notifier.xmlHttpReq = new XMLHttpRequest();
		}
		gYm_notifier.xmlHttpReq.open("GET", gYm_notifier.checkURL, true);
		gYm_notifier.xmlHttpReq.channel.loadFlags |= Ci.nsIRequest.LOAD_BYPASS_CACHE;
		gYm_notifier.workAroundBug437174(gYm_notifier.xmlHttpReq);
		gYm_notifier.privateBrowsingCheck(gYm_notifier.xmlHttpReq);
		gYm_notifier.xmlHttpReq.setRequestHeader('Cache-Control','no-cache, must-revalidate, proxy-revalidate');
		gYm_notifier.xmlHttpReq.setRequestHeader('Pragma','no-cache, must-revalidate, proxy-revalidate');
		gYm_notifier.xmlHttpReq.onerror = function() {
			// An error occurred, notify user
			gYm_notifier.ym_log("checkmail Onerror fired with status of " + gYm_notifier.xmlHttpReq.status +
				                    " and readyState of " + gYm_notifier.xmlHttpReq.readyState);
			gYm_notifier.xmlHttpReq.abort();
			gYm_notifier.xmlHttpReq = null;  // needed or send won't do anything
			gYm_notifier.checkFailCount = gYm_notifier.checkFailCount + 1;
			if (gYm_notifier.checkFailCount > 3) {
				gYm_notifier.checkFailCount = 0;
				var observerService = Cc["@mozilla.org/observer-service;1"].getService(Ci.nsIObserverService);
				observerService.notifyObservers(null,"ym-gui-change","check-error " + username);
			}
			else {
				gYm_notifier.checkMail();
			}
		}
		gYm_notifier.xmlHttpReq.onload = function() {
			gYm_notifier.ym_log('request status (' + gYm_notifier.xmlHttpReq.status + ')');
			if (gYm_notifier.xmlHttpReq.readyState==4) {
				gYm_notifier.checkFailCount = 0;
				if (gYm_notifier.xmlHttpReq.status == 200) {
					var response = gYm_notifier.xmlHttpReq.responseText;
					gYm_notifier.ym_log('Response: ' + response);
					var result = null;
					if (gYm_notifier.checkURL == gYm_notifier.defaultcheckURL) {
						try {
							var json_object = JSON.parse(response);
							var channels = json_object[0].data.channels["/user/ymail"];
							result = (channels) ? channels.data : null;
						} catch(ex) {
							gYm_notifier.ym_logError(ex, 1);
						}
/* - Old widget check
						var xmldoc = gYm_notifier.xmlHttpReq.responseXML;
						var description = xmldoc.getElementsByTagName('description').item(0);
						if ((typeof(description) != "undefined") && description && description.firstChild) {
							result = description.firstChild.data;
							gYm_notifier.ym_log('Result: ' + result);
						}
						// not logged in 
						else {
							gYm_notifier.ym_log('Checking returned no data, try logging in again.')
							gYm_notifier.doLogin(true);
							return;
						}
*/
					}
					else if (gYm_notifier.checkURL == "http://www.yahoo.co.jp") {
						if (/<li class="pbmailinfo">.+(\d+).+<\/li>/.test(response)) result = RegExp.$1;
							gYm_notifier.ym_log('Result: ' + result);
					}
	
					var now = new Date();
					var hours = now.getHours();
					var minutes = now.getMinutes();
					if (minutes < 10) minutes = "0" + minutes;
					var lastCheckTime = "(" + hours + ":" + minutes + ")";
	
					try {
						if (!result) {
							gYm_notifier.ym_log("no mail");
							gYm_notifier.alertUser(0,username,lastCheckTime);
						}
						else {
							gYm_notifier.ym_log("new mail: " + result + " messages");
						
							// Alert the user if needed
							gYm_notifier.alertUser(result,username,lastCheckTime);
		 
							//	Allow user to override
							// TBD MULTI-USER SUPPORT
							ym_prefs.loadUser(null, false);
							if (ym_prefs.user.userName && !ym_prefs.user.overrideURL) {
								ym_prefs.saveUserMailURL(gYm_notifier.mailURL);
							}
						}
					}
					catch(ex) {
						gYm_notifier.ym_logError(ex, 1);
					}
				}
				else {
					gYm_notifier.ym_log("Bad status, showing error\n" + gYm_notifier.xmlHttpReq.getAllResponseHeaders());
					gYm_notifier.ym_write_log((new Date).toGMTString() + ':\nResponse: ' + gYm_notifier.xmlHttpReq.responseText + '\n');
					var observerService = Cc["@mozilla.org/observer-service;1"].getService(Ci.nsIObserverService);
					observerService.notifyObservers(null,"ym-gui-change","check-error " + username);
				}
			}
		}
		// If xmlHttpReq is ready, send request
		if (gYm_notifier.xmlHttpReq.readyState == 1) {
			// if retrying clear pending retry
			if (gYm_notifier.sendRetry2Id) {
				clearTimeout( gYm_notifier.sendRetry2Id );	 // Clear pending retry  
				gYm_notifier.sendRetry2Id = null;
			} 		
			gYm_notifier.xmlHttpReq.send(null);
		}
		// if it is not ready try again in 2 seconds
		else {
			gYm_notifier.ym_log("Checkmail readyState = " + gYm_notifier.xmlHttpReq.readyState + ", retrying in 2 seconds.");
			gYm_notifier.xmlHttpReq.abort();
			gYm_notifier.sendRetry2Id = setTimeout(gYm_notifier.checkMail, 2000);
		}
	},
	
	//
	// Cookie functions to determine if user is logged in or not
	// This will not tell what username user is logged in under though
	//
	// Returns:
	//	 0 if no user logged in
	//	 1 if logged in as different user
	//	 2 if logged in as correct user for browser session, but we want to verify login
	//	 3 if permanently logged in logged in as correct user, but we want to verify login 
	//	 4 if logged in as correct user for browser session
	//	 5 if permanently logged in logged in as correct user 
	//
	loginState : function(username) {
	
		ym_prefs.loadUser(username, false);
		
		// set correct URLs if there is an actual username
		if (ym_prefs.user.userName) this.findYahooURL(ym_prefs.user.userName);
	
		var cookie = this.getCookie('.' + this.cookieDomain, 'Y');
		if (!cookie) return 0;
		
		var value = cookie.value;
		if (!value) return 0;
		
		// 'p' and 'l' will never change for a username, 'n' will change if password changes
		var cookieData = this.decodeQueryString(value);
		if (!cookieData) return 0;
		if (!cookieData['l'] || cookieData['l'] == '') return 0;
		this.ym_log(JSON.stringify(cookieData));

		var rc;
		if (ym_prefs.user.userName && ym_prefs.user.cookie) {
			var userCookie = this.decodeQueryString(ym_prefs.user.cookie)
			if (userCookie && (cookieData['l'] == userCookie['l']) && (cookieData['p'] == userCookie['p'])) {
				if (cookieData['n'] == userCookie['n']) {
					if (ym_prefs.user.useCookie) {
						if ((cookieData['np'] || 0) == 1) rc = 4;
						else rc = 5;
					}
					else {
						if ((cookieData['np'] || 0) == 1) rc = 2;
						else rc = 3;
					}
				}
				else {
					// n's aren't equal at this point which means password was changed so clear stored cookie
					// and return 1
					this.ym_log(username + " changed password, clearing stored cookie.");
					ym_prefs.saveUserCookie("", username);
					rc = 1;
				}
			}
			else rc = 1;
		}
		else rc = 1;
			
		return rc;
	},

	decodeQueryString : function(qs) {
		qs = qs.replace(/%26/g, '&');
		qs = qs.replace(/%3D/g, '=');
		qs = qs.replace(/%25/g, '%');
		qs = qs.replace(/%3F/g, '?');
		var queryHash = new Array();
		var pairs = qs.split('&');
		for(var i = 0; i < pairs.length; i++) {
			var split = pairs[i].split('=');
			var key = split.shift();
			var val = split.join('=');
			queryHash[key] = val;
		}
		return queryHash;
	},
	
	getCookie : function(domain, name) {
	
		// If window is in private mode in Firefox 19 and up, cookie will be null so use our cookie if available
		if ((this.PrivateBrowsingUtils && this.PrivateBrowsingUtils.isWindowPrivate(window)) || (this.PrivateBrowsing && this.PrivateBrowsing.autoStarted))  
			return { name: name, value : this.cookieValue[name] };
		else {
			var cmClass = "@mozilla.org/cookiemanager;1";
			var cms = Cc[cmClass].getService();
			var cm = cms.QueryInterface(Ci.nsICookieManager);
			var enumerator = cm.enumerator;
			var cIface = Ci.nsICookie;
			var domainRegex = new RegExp(domain.replace('\.', '\\.')+'$');
			while (enumerator.hasMoreElements()) {
				var nextCookie = enumerator.getNext().QueryInterface(cIface);
				if (nextCookie.host.match(domainRegex)) {
					if (nextCookie.name == name) return nextCookie;
				}
			}
			return null;
		}
	},
	
	//
	// alert the user if needed depending on the preferences
	//
		alertUser : function(nbMessages, username, lastCheckTime) {
		var nbMessagesInt = parseInt(nbMessages);
		
		var observerService = Cc["@mozilla.org/observer-service;1"].getService(Ci.nsIObserverService);
		observerService.notifyObservers(null,"ym-gui-change",nbMessages + " " + username + " " + lastCheckTime);
	
		if (nbMessagesInt > 0) {
			
			// Check to see if new mail count has changed to see if there really is new mail
			var trueNewMail = false;
			if (this.newMailCount != nbMessagesInt) trueNewMail = true;
			this.newMailCount = nbMessagesInt;
			
			if ((ym_prefs.getPref(ym_prefs.PREF_SOUND) == ym_prefs.CONST_REPEAT) ||
				((ym_prefs.getPref(ym_prefs.PREF_SOUND) == ym_prefs.CONST_ONCE) && trueNewMail)) {
				var soundUrl = decodeURIComponent(ym_prefs.getPref(ym_prefs.PREF_SOUNDFILE));
				var soundOk = false;
	
				// store sound instance in main object since sound will stop playing
				// if instance is destroyed.
				try {
					if (!this.sound) this.sound = Cc["@mozilla.org/sound;1"].createInstance(Ci.nsISound);
					soundOK = true;
				}
				catch (e) {}
				
				if (soundOK) {
	
					var failed = true;
					var FileFactory = null;
					var soundFile = null;
					var aURI = false;
	
					// play default system mail sound if nothing specified
					if (soundUrl=="") {
						if (this.Mozilla_1_9_2_plus)
							this.sound.playEventSound(Ci.nsISound.EVENT_NEW_MAIL_RECEIVED); 
						else
							this.sound.playSystemSound("_moz_mailbeep"); 
						failed = true;
					}
					// if a URL use that
					else if (soundUrl.indexOf("://") != -1) {
						soundFile = soundUrl;
						aURI = true;
					}
					// must be a filename (local or absolute)
					else {
						failed = true;	// default to fail
						try {
							FileFactory = new Components.Constructor('@mozilla.org/file/local;1', 'nsILocalFile', 'initWithPath');
							soundFile = new FileFactory(soundUrl);
							failed = false;
						} 
						catch (e) {}
	
						// if fail try to use a relative path in the chrome directory
						if (failed) {
							try {
								// get user chrome directory ( see http://kb.mozillazine.org/File_IO#Getting_special_files )
								var file = Cc["@mozilla.org/file/directory_service;1"].getService(Ci.nsIProperties).get("UChrm", Ci.nsIFile);
								file.append(soundUrl);
								FileFactory = new Components.Constructor('@mozilla.org/file/local;1', 'nsILocalFile', 'initWithFile');
								soundFile = new FileFactory(file);
								failed = false;
							} 
							catch(ex) { 
								this.ym_logError(ex, 1);
							}
						}
					}
					if (!failed) {
						try {
							var soundURI = null;
							var nsIIOService = Cc['@mozilla.org/network/io-service;1'].getService(Ci.nsIIOService);
							if (aURI) soundURI = nsIIOService.newURI(soundFile,null,null);
							else soundURI =  nsIIOService.newFileURI(soundFile);
							this.sound.play(soundURI);
						} 
						catch(ex) { 
							this.ym_logError(ex, 1);
						}
					}
				}
			}
	
			// TBD MULTI-USER SUPPORT
			var titleText = this.getString("new") + " Yahoo! Mail (" + ym_prefs.getPref(ym_prefs.PREF_DEFAULT_USER) + ")";
			var textStr = this.getString("youhave") +" " + nbMessages + " ";
			if (nbMessagesInt == 1) textStr = textStr + this.getString("message");
			else textStr = textStr + this.getString("messages");
	
			if ((ym_prefs.getPref(ym_prefs.PREF_SHOW_ALERTS) == ym_prefs.CONST_REPEAT) ||
					((ym_prefs.getPref(ym_prefs.PREF_SHOW_ALERTS) == ym_prefs.CONST_ONCE) && trueNewMail)) {
				try {
					// Use Growl Notifications Addon if available (only for FF2, in FF3 growl uses nsIAlertsService)
					if (Ci.grINotificationsList) {
						var grn = Cc["@growl.info/notifications;1"].getService(Ci.grINotifications);
						grn.sendNotification("Yahoo Mail Notifier", "chrome://ym_notifier/skin/newmail.png", titleText, textStr, "", this);
					}
					else {
						var alerts = Cc["@mozilla.org/alerts-service;1"].getService(Ci.nsIAlertsService);
						alerts.showAlertNotification("chrome://ym_notifier/skin/newmail.png", titleText, textStr, true, "", this);
					} 
				}
				catch(ex) { 
					this.ym_logError(ex, 1);
				}
			}
	
			if ((ym_prefs.getPref(ym_prefs.PREF_SHOW_POPUP) == ym_prefs.CONST_REPEAT) ||
				((ym_prefs.getPref(ym_prefs.PREF_SHOW_POPUP) == ym_prefs.CONST_ONCE) && trueNewMail)) {
				alert (titleText + "\n" + textStr);
			}
		}
		else {
			gYm_notifier.newMailCount = 0;
		}
	},

	//
	// Observer functions
	//  
	observe : function(aSubject, aTopic, aData) {
		
		this.ym_log("Observer callback - aSubject : " + aSubject + ", aTopic : " + aTopic + ", aData : " + aData);
							
		switch (aTopic) {
			// alert went away by itself
			case "alertfinished":
				break;
	
			// user clicked alert 									
			case "alertclickcallback":
				this.clickAlert();
				break;
	
			// user logged in or out or change of number of new messages
			case "ym-gui-change":
				this.updateGUI(aData);
				break;
	
			// preferences changed
			case "nsPref:changed":
	
				switch(aData) {
					
					// checking interval change
					case ym_prefs.PREF_INTERVAL: 
					
						if (this.timer || ym_prefs.getPref(ym_prefs.PREF_AUTO_CHECK))
							this.startChecking();
						break;
					
					case ym_prefs.PREF_STATUSBAR:
					
						var myStatusbar = document.getElementById("ym-notifier-statusbar-item");
						if (myStatusbar) {
							var showStatusbar = ((ym_prefs.getPref(ym_prefs.PREF_STATUSBAR) == ym_prefs.CONST_ALWAYS) ||
												 ((ym_prefs.getPref(ym_prefs.PREF_STATUSBAR) == ym_prefs.CONST_ON_NEW_MAIL) && 
													(myStatusbar.getAttribute("newMail") == true)));
							myStatusbar.setAttribute("collapsed", !showStatusbar);
						}
						break;
				}
				break;
	
			// timer periodic call
			case "timer-callback":
	
				this.checkMail();
				break;
				
			case "cookie-changed":
			case "private-cookie-changed":
				let cookie = aSubject.QueryInterface(Components.interfaces.nsICookie2);
				// host has a "." in front of it
				if (((cookie.host == ".yahoo.com") || (cookie.host == "yahoo.co.jp")) && ((cookie.name == "Y") || (cookie.name == "T"))) {
					this.cookieValue[cookie.name] = cookie.value;
				}
				break;
				
		}
	},

	// nsIAlertsService - nsIAlertListener was deprecated and replaced with nsIObserver
	//										This is to maintain 1.7 compatability.
	onAlertFinished : function(aCookie) {},
	onAlertClickCallback : function(aCookie) {
	 	this.clickAlert();
	},

	//
	// Update statusbar and toolbar
	//
	updateGUI : function(aData) {
		var myStatusbar = document.getElementById("ym-notifier-statusbar-item");
		var myToolbar = document.getElementById("ym-notifier-toolbar-item");
		var myLogout = document.getElementById("ym-notifier-context-menu-logout");
		var myMenu = document.getElementById("ym-notifier-context");
	
		aData = aData.split(" ");
		
		var loginState = this.loginState(aData[1]);
		var checkerror = (aData[0] == "check-error");
		var justloggedin = ((aData[0] == "login") && (loginState > 0));
		var loggedin = ((aData[0] != "logout") && ((loginState > 1) || justloggedin));
		var mailChange = ((aData[0] != "login") && (aData[0] != "logout"));
		var newMail = (parseInt(aData[0]) > 0); 
		var defaultUser = ym_prefs.getPref(ym_prefs.PREF_DEFAULT_USER);
		var myTooltip;
		
		this.ym_log("loginState : " + loginState + ", checkerror : " + checkerror +
					", loggedin : " + loggedin + ", justloggedin : " + justloggedin +
					", mailChange : " + mailChange + ", newMail : " + newMail + 
					", defaultUser : " + defaultUser + ", aData[0] : " + aData[0] + 
					", aData[1] : " + aData[1] + ", aData[2] : " + aData[2]);

		// errors take highest preference
		if (checkerror) {
			myTooltip = aData[1] + ": " + this.getString("checkerror");
			newMail = false;
			aData[0] = "";
			mailChange = true;
		}
		// logging out takes preference over everything except errors
		else if (!loggedin) {
			this.stopChecking();
			myTooltip = this.getString("nologin");
			newMail = false;
			mailChange = true;
			aData[0] = "";
			this.newMailCount = 0;
		}
		else {
			// if default user just logged in  // TBD MULTI-USER SUPPORT
			if (justloggedin &&	(aData[1] == defaultUser)) {
	
				// set correct URLs
				this.findYahooURL(aData[1]);
				// store updated login and name part of the login cookie
				var cookie = this.getCookie('.' + this.cookieDomain, 'Y');
				ym_prefs.saveUserCookie(cookie.value);
	//			var cookieData = this.decodeQueryString(cookie.value);
	//			ym_prefs.saveUserCookie('l=' + cookieData['l'] + "&n=" + cookieData['n'] +
	//									'&p=' + cookieData['p'], aData[1]);
	
				// only start checking on login if not already checking
				if (!this.timer) this.startChecking();
				else {
					// Workaround for https://bugzilla.mozilla.org/show_bug.cgi?id=174320 in older browsers
					if (this.oldBrowser) window.setTimeout(gYm_notifier.checkMail, 0)
					else gYm_notifier.checkMail();
				}
			}

			if (newMail) {
				myTooltip = this.getString("youhave") + " " + aData[0] + " ";
				if (parseInt(aData[0]) == 1) myTooltip = myTooltip + this.getString("message");
				else myTooltip = myTooltip + this.getString("messages");
			}
			else {
				myTooltip = this.getString("nomail");
			}  
		}
	
		// if just checked for mail, add time to tooltip otherwise grab last checked 
		// time from statusbar attribute
		if (aData[2] && aData[2]!="undefined") myTooltip = myTooltip + " " + aData[2];
		else if (myStatusbar && loggedin) 
			myTooltip = myTooltip + " " + myStatusbar.getAttribute("lastCheckTime");
				
		if (myStatusbar) {
			myStatusbar.setAttribute("logged-in", loggedin);
			myStatusbar.setAttribute("error", checkerror);
			myStatusbar.setAttribute("checking", (aData[2] && aData[2]!="undefined"));
			if (mailChange) {
				myStatusbar.setAttribute("tooltiptext", myTooltip);
				myStatusbar.setAttribute("newMail", newMail);
	
				// do not change current count on error
				if (!checkerror) {
					myStatusbar.setAttribute("label", aData[0]);
					if (loggedin) myStatusbar.setAttribute("lastCheckTime", aData[2]);
					else myStatusbar.setAttribute("lastCheckTime", "");
				}
	
				// show or hide status bar if setting is set to only show on new mail
				if (ym_prefs.getPref(ym_prefs.PREF_STATUSBAR) == ym_prefs.CONST_ON_NEW_MAIL)
					myStatusbar.setAttribute("collapsed", !newMail);
			}
		}
		if (myToolbar) {
			myToolbar.setAttribute("logged-in", loggedin);
			myToolbar.setAttribute("error", checkerror);
			if (mailChange) {
				myToolbar.setAttribute("tooltiptext", myTooltip);
				myToolbar.setAttribute("newMail", newMail);
	
				// do not change current count on error
				if (!checkerror) myToolbar.setAttribute("unread", aData[0]);
			}
		}
		if (myLogout) myLogout.setAttribute("disabled", !loggedin);
		
		// hide the context popup menu if it is displayed
		if (myMenu) myMenu.hidePopup();
		
		// clear aData
		aData = null;
	} ,

	//
	// Go to mail url if user clicks alert window.	Since user can only left click an alert, 
	//  this will only open in a new tab or window based off the single click (or double click
	// if single click is not set or set to same tab) preference. This will never overwrite 
	// the current tab since user might not want to do so.
	//
	clickAlert : function() {
		//If user clicks alert on mail in new tab
		try {
			var wm = Cc["@mozilla.org/appshell/window-mediator;1"].getService(Ci.nsIWindowMediator);
			var top = wm.getMostRecentWindow("navigator:browser");
			if (!top)
				this.gotoYahooMail(ym_prefs.CONST_NEWWIN, null);
			else {
				var option = ym_prefs.getPref(ym_prefs.PREF_CLICK);
				if ((option == ym_prefs.CONST_NOTHING) || (option == ym_prefs.CONST_SAMETAB) ||
						(option >= ym_prefs.CONST_CHECKMAIL))
					option = ym_prefs.getPref(ym_prefs.PREF_DOUBLECLICK);
				if ((option == ym_prefs.CONST_NOTHING) || (option == ym_prefs.CONST_SAMETAB) ||
						(option >= ym_prefs.CONST_CHECKMAIL))
					option = ym_prefs.CONST_FOCUSTAB;
				this.gotoYahooMail(option, top.document.getElementById("content"));
			}
		}
		catch(ex) {
			// There are no browser windows open and therefore no component services available (no preferences)
			// so open a new window and go to this.mailURL as a last ditch effort
			window.openDialog("chrome://browser/content/browser.xul", "_blank",
								"chrome,all,dialog=no", this.mailURL, null, null);
		}
	},

	// 
	// handles single, double and middle button mouse clicks
	//
	handleClick : function(event) {
		// ignore clicks with ctrl or alt key pressed
		if ((event.ctrlKey)||(event.altKey)) return false;
	
		// for our purposes a double click is 2 single clicks
		if (event.type == "click") {
			var eShiftKey = ((!(event.ctrlKey)||(event.metaKey)) && (!event.altKey) && (event.shiftKey));
			var midMouse = ((event.button == 1) || ((event.button == 0) && (event.shiftKey)))
							 	
			// if not a left click, middle mouse click or shift click return
			if ((event.button != 0) && !midMouse) return false;
				 	
			if (midMouse) { 
				this.gotoYahooMail(ym_prefs.getPref(ym_prefs.PREF_MIDDLECLICK), document.getElementById("content"));
			}
			else {
				var dcTime = ym_prefs.getPref(ym_prefs.PREF_DOUBLECLICK_TIME);
	
				// If we've just had a click then this is a double click
				if (this.mouseId) {
					clearTimeout( this.mouseId ); 	// Clear pending Click	
					this.mouseId = null;
					this.gotoYahooMail(ym_prefs.getPref(ym_prefs.PREF_DOUBLECLICK), document.getElementById("content"));
				}
				// first click, might be start of double click so wait a few ms before processing
				else {
					var d = new Date();
					var now = d.getTime();
	
					this.mouseEvtTime = now;
					this.mouseId = setTimeout(function() { gYm_notifier.doClick(ym_prefs.getPref(ym_prefs.PREF_CLICK)); }, dcTime);
				}
			}
		}
		return false;
	},

	// 
	// this handles single clicks that aren't actually double clicks
	//
	doClick : function(clickPref) {
		this.mouseId = null;
		this.gotoYahooMail(clickPref, document.getElementById("content"));
		return false;
	},
		

	// 
	// opens Yahoo! Mail based off of user clicking preferences
	//
	gotoYahooMail : function(option, tBrowser) {
		// read stored mailurl or use default if none stored
		ym_prefs.loadUser(null, false);					 // TBD MULTI-USER SUPPORT
		
		var mailURL = this.mailURL;
		if (ym_prefs.user.userName) {
		 	// find the correct default URL
			this.findYahooURL(ym_prefs.user.userName);
			if (ym_prefs.user.overrideURL && (ym_prefs.user.mailURL != "")) mailURL = ym_prefs.user.mailURL;
		}
		
		// if option is greater than CONST_CHECKMAIL then user wants to check mail on click
		if (option >= ym_prefs.CONST_CHECKMAIL) this.manualCheckMail();
		else {
		
			// Is current tab blank.	Different methods for determining depending on whether browser is
			// FF 1.5 or FF 1.0 based
			var currBlank =  (tBrowser && 
			                  (tBrowser.mCurrentTab.linkedBrowser && 
			                   (tBrowser.mCurrentTab.linkedBrowser.contentDocument.location == "about:blank")) ||
			                  (!tBrowser.mCurrentTab.linkedBrowser && 
			                   (tBrowser.mCurrentTab.label == "(Untitled)")));
											 
			// If using FF 1.5 or greater, try and find if Yahoo Mail is open in a tab and just switch there.
			if (tBrowser && tBrowser.mCurrentTab.linkedBrowser) {
				for (var i=0; i<tBrowser.mTabs.length; i++) {
					var tablocation = String(tBrowser.mTabs[i].linkedBrowser.contentDocument.location).toLowerCase();
					// find a Yahoo Mail tab that isn't being used for composing
					if ((tablocation.indexOf("mail.yahoo") != -1) && (tablocation.indexOf("/ym/compose") == -1)) {
						tBrowser.selectedTab = tBrowser.mTabs[i];
						// if not Yahoo Mail Classic don't reload
						if (tablocation.indexOf("/mc/") != -1) {
							option = ym_prefs.CONST_SAMETAB;
						}
						else {
							option = ym_prefs.CONST_NOTHING;
						}
						break;
					}
				}
			}
		 
			switch (option) {
				case ym_prefs.CONST_NOTHING:
					return;
					break;
							
				case ym_prefs.CONST_SAMETAB:
					tBrowser.loadURI(mailURL);
					break;
							 	
				case ym_prefs.CONST_NEWTAB:
					if (currBlank) tBrowser.loadURI(mailURL);
					else tBrowser.addTab(mailURL);
					break;
							 	
				case ym_prefs.CONST_FOCUSTAB:
					if (currBlank) tBrowser.loadURI(mailURL);
					else {
						var tab = tBrowser.addTab(mailURL);
						tBrowser.selectedTab = tab;
					}
					break;
							 	
				case ym_prefs.CONST_NEWWIN:
					window.open(mailURL, "", "");
					break;
			}

			// set icons and text to no new mail if checking for mail
			var myStatusbar = document.getElementById("ym-notifier-statusbar-item");
			if (myStatusbar && myStatusbar.getAttribute("checking")) this.alertUser(0);
		}
	},

	//
	// Open the preferences window
	// 
	openPrefWindow : function() {

		var wm = Cc['@mozilla.org/appshell/window-mediator;1'].getService(Ci.nsIWindowMediator);
		var dialog = wm.getMostRecentWindow("YMNotifier:Options");
		if (dialog)
		{
			dialog.focus();
			return;
		}
				
		var prefWindow = window.openDialog('chrome://ym_notifier/content/ym_prefs.xul', 'ym_pref_window', 
											'close=no, dialog, chrome, modal, centerscreen');
	},

	//
	// Open the login window
	// 
	openLoginWindow : function() {

		var wm = Cc['@mozilla.org/appshell/window-mediator;1'].getService(Ci.nsIWindowMediator);
		var dialog = wm.getMostRecentWindow("YMNotifier:Login");
		if (dialog)
		{
			dialog.focus();
			return;
		}
				
		var loginWindow = window.openDialog('chrome://ym_notifier/content/ym_login.xul', 'ym_login_window', 
											'close=no, dialog, chrome, modal, centerscreen');
	},

	//
	// Get the localized strings
	//
	getString : function(stringName) {
		var strings = document.getElementById("ym-notifierStrings");
		return strings.getString(stringName);
	},

	//
	// Utility to create an error message in the log without throwing an error.
	//
	ym_logError : function(e, force) {
		try { 
			if (force || ym_prefs.getPref(ym_prefs.PREF_LOG_ENABLED)) {
				var consoleService = Cc['@mozilla.org/consoleservice;1'].getService(Ci.nsIConsoleService);

				consoleService.logStringMessage("ym_notifier (" + (new Date).toGMTString() + "): {" + e.message + "} {" + e.location + "}");
				this.ym_write_log((new Date).toGMTString() + ": {" + e.message + "} {" + e.location + "}" + "\n");
			}
		}
		catch (ex) {
			dump (ex + "\n")
		}
	},

	//
	// Log info messages
	//
	ym_log : function(aMessage, force) {
		try {
			if (force || ym_prefs.getPref(ym_prefs.PREF_LOG_ENABLED)) {
				var consoleService = Cc['@mozilla.org/consoleservice;1'].getService(Ci.nsIConsoleService);
				consoleService.logStringMessage("ym_notifier (" + (new Date).toGMTString() + "): " + aMessage);
				this.ym_write_log((new Date).toGMTString() + "): " + aMessage + "\n");
			}
		}
		catch (ex) { 
			dump(ex + "\n"); 
		}
	},

	// 
	// Delete Log File if it exists and not logging or it's too large (> 1 MB)
	//
	deleteLogFile : function() {
		try { 
			var logFile = Cc["@mozilla.org/file/directory_service;1"].getService(Ci.nsIProperties).get("ProfD", Ci.nsILocalFile).clone();
			logFile.append(this.logFileName);

			if (logFile.exists() && (!ym_prefs.getPref(ym_prefs.PREF_LOG_ENABLED) || logFile.fileSize > 1048576)) {
				logFile.remove(false);
			}
		}
		catch (ex) { 
			dump(ex + "\n"); 
		}
	},
				 
	//
	// Write to Log File
	// 
	ym_write_log : function(aMessage) {
		try {
			if (!this.logFile) {
				this.logFile = Cc["@mozilla.org/file/directory_service;1"].getService(Ci.nsIProperties).get("ProfD", Ci.nsILocalFile).clone();
				this.logFile.append(this.logFileName);
			}
		
			var stream = Cc["@mozilla.org/network/file-output-stream;1"].createInstance(Ci.nsIFileOutputStream);
			// ioFlags: write only, create file, append;	Permission: read/write owner
			stream.init(this.logFile, 0x02 | 0x08 | 0x10, 0600, 0);
			var cvstream = Cc["@mozilla.org/intl/converter-output-stream;1"].createInstance(Ci.nsIConverterOutputStream);
			cvstream.init(stream, "UTF-8", 0, Ci.nsIConverterInputStream.DEFAULT_REPLACEMENT_CHARACTER);
			
			cvstream.writeString(aMessage.replace(/[\n$]/g, this.mEOL));
			cvstream.flush();
			cvstream.close();
		}
		catch (ex) { 
			dump(ex + "\n"); 
		}
	},

	//
	// Uninstall procedure
	//
	uninstall : function() {
		if (!this.isMozillaOrSeaMonkey1()) return;
	
		var confirmUninstall = confirm(this.getString("confirmuninstall"));
		
		// user choose to cancel
		if (!confirmUninstall) return; // do nothing further
		
		// disable the "Uninstall" menuitem
		document.getElementById("ym-notifier-context-menu-uninstall").setAttribute("disabled", true); 
		
		var oExtUninstall = new ExtensionUninstall("urn:mozilla:package:ym_notifier");
		var success = oExtUninstall.uninstall();
		this.uninstalled(oExtUninstall);
	},

	//
	// Output Result of Uninstall
	//
	uninstalled : function(oExtUninstall) {
		var message;
		
		if(oExtUninstall != null && oExtUninstall.hasError())
			message = this.getString("uninstallerror");
		else if(oExtUninstall != null && oExtUninstall.hasWarning())
			message = this.getString("uninstallwarning");
		else
			message = this.getString("uninstallsuccess");
		
		var prompts = Cc["@mozilla.org/embedcomp/prompt-service;1"].getService(Ci.nsIPromptService);
		var flags = prompts.BUTTON_TITLE_IS_STRING * prompts.BUTTON_POS_0 + 
					prompts.BUTTON_TITLE_IS_STRING * prompts.BUTTON_POS_1;
		var check = {value: false};
		var button = prompts.confirmEx(window, this.getString("uninstalldone"), message, flags, 
		                               this.getString("closebutton"), this.getString("viewlogbuton"), null, null, check);
		if (button) oExtUninstall.showLogDialog(true, self);
	},

	//
	// Only display uninstall for Mozilla and SeaMonkey versions less than 2.0
	// 
	showUninstall : function() {
		if (this.isMozillaOrSeaMonkey1()) {
			document.getElementById('ym-notifier-context-menu-uninstall').removeAttribute('hidden');
		}
	},
	
	isMozillaOrSeaMonkey1 : function() {
		var id = null, version = 0;
		if("@mozilla.org/xre/app-info;1" in Components.classes) {
			// running under Mozilla 1.8 or later
			var appInfo = Components.classes["@mozilla.org/xre/app-info;1"].getService(Components.interfaces.nsIXULAppInfo);
			id = appInfo.ID;
			version = appInfo.version;
		} else {
			try {
				var pref = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefBranch);
				id = pref.getCharPref("app.id");
				version = pref.getCharPref("app.version");
			} catch(e) {
				// very old version
			}
		}

		// SeaMonkey 1.x.x, Mozilla or really old version
		if (!id || ((id == "{92650c4d-4b8e-4d2a-b7eb-24ecf4f6b63a}") && (parseInt(version) < 2)) ||
		    (id == "{86c18b42-e466-45a9-ae7a-9b95ba6f5640}")) {
			return true;
		}
		else return false;
	},

	workAroundBug437174 : function(request) {
		if (ym_prefs.getPref("network.cookie.cookieBehavior", true) == 1) {
			// work around bug 437174 if necessary
			if (this.Mozilla_1_9) {
				var ds = Cc["@mozilla.org/webshell;1"].createInstance(Ci.nsIDocShellTreeItem).QueryInterface(Ci.nsIInterfaceRequestor);
				ds.itemType = Ci.nsIDocShellTreeItem.typeContent;
				request.channel.loadGroup = ds.getInterface(Ci.nsILoadGroup);
				request.channel.loadFlags |= Ci.nsIChannel.LOAD_DOCUMENT_URI;
			}
			else {
				// Force to allow third party cookies (only works in Firefox 3.6 (1.9.2) and above)
				var hci = request.channel.QueryInterface(Ci.nsIHttpChannelInternal);
				if (typeof(hci.forceAllowThirdPartyCookie) == "boolean") {
					hci.forceAllowThirdPartyCookie = true;
				}
				else {
					// No choice in Firefox 3.5 other than to add a cookie exception
					// Try and find cookie exceptions for yahoo.com or yahoo.co.jp
					try {
						var pManager = Cc["@mozilla.org/permissionmanager;1"].getService(Ci.nsIPermissionManager);
						var ioService = Cc['@mozilla.org/network/io-service;1'].getService(Ci.nsIIOService);

						var url = ioService.newURI("http://" + this.cookieDomain, null, null);

						// get existing permission if it exists
						var permission = pManager.testPermission(url, 'cookie');
						// if not allowed, add an allow exception
						if ((permission != 1) && (permission != 8)) {
							pManager.add(url, 'cookie', 1);
							this.ym_log("Adding an allow cookies exception for " + this.cookieDomain);
						}
					} catch (ex) {
						dump(ex + "\n"); 
						this.ym_logError(ex, 1);
					}
				}
			}
		}
	}, 
	
	privateBrowsingCheck : function(request) {
		// need to use private channel if permanent private browsing is enabled
		if ((this.PrivateBrowsingUtils && this.PrivateBrowsingUtils.isWindowPrivate(window)) || (this.PrivateBrowsing && this.PrivateBrowsing.autoStarted)) {
			try {
				let channel = request.channel.QueryInterface(Components.interfaces.nsIPrivateBrowsingChannel);
				channel.setPrivate(true);
			}
			catch(ex) {}
		}
	}
}

XPCOMUtils.defineLazyGetter(gYm_notifier, "PrivateBrowsing", function() {
	if (Cc["@mozilla.org/privatebrowsing;1"]) {
		XPCOMUtils.defineLazyServiceGetter(gYm_notifier, "PrivateBrowsing", "@mozilla.org/privatebrowsing;1", "nsIPrivateBrowsingService");
		return PrivateBrowsing;
	}
	else 
		return null;
});

// This only exists in Gecko 17 and up, so define our own lazy getter to set it to null if it doesn't exist instead of throwing an exception
XPCOMUtils.defineLazyGetter(gYm_notifier, "PrivateBrowsingUtils", function() {
	XPCOMUtils.defineLazyModuleGetter(gYm_notifier, "PrivateBrowsingUtils", "resource://gre/modules/PrivateBrowsingUtils.jsm");
	try {
		return PrivateBrowsingUtils;
	}
	catch(ex) {
		return null
	}
});

gYm_notifier.init();
window.addEventListener('load', gYm_notifier.initialize, false);
